declare module BA.Ui.Duplicates {
    /**
     * The global service that registers field changes and may search and notify for duplicates of the currently
     * edited record.
     */
    class DuplicateService {
        /**
         * A 'map' containing all changes that have been registered for a given record (key).
         * The contained data is maintained by closing tabs and dialogs.
         *
         * Key: RecordID (string, lower case expected)
         * Value: Array of DuplicateChangeInfo
         */
        private Changes;
        /** Handle for the interval that is started when a live duplicate search is needed. */
        private CheckIntervalHandle;
        /** True, if the live check for duplicates is enabled */
        private LiveCheckEnabled;
        /** Constructor of the service. Initializes member variables. */
        constructor();
        /**
         * Function for registering a changed value of some relevant field.
         *
         * @param mainRecordGuid The oid of the main record that this change is related to
         * @param mainRecordDataSource The data source guid of the main record that this change is related to
         * @param currentRecordId The oid of the record that has been changed
         * @param dataSource The data source guid of the record that has been changed
         * @param elementName The Field name or the type guid of a relation that has been changed
         * @param value The new value of the field or the relation (an oid then)
         * @param temporary Used only on relations. True if the relation in question has not yet been persisted.
         */
        RegisterChange(mainRecordGuid: BA.Classes.Guid, mainRecordDataSource: BA.Classes.Guid, currentRecordId: BA.Classes.Guid, dataSource: BA.Classes.Guid, elementName: string | BA.Classes.Guid, value: string, temporary: boolean): void;
        /**
         * Retrieves the changed data in form of models that are essentially the same as the DuplicateInfo-models but
         * lack the functionality to initiate a live check.
         *
         * @param mainRecordGuid the main record oid to get the changes for
         * @returns An array of the current change information for transferring that information to the server
         */
        GetTransferModels(mainRecordGuid: BA.Classes.Guid): DuplicateChangeInfoTransferModel[];
        /**
         * Removes the logged changes for the given main record and optionally the given record id in these changes.
         * The function provides the possiblity to not remove a relation but switch it to not-temporary-anymore
         * instead.
         *
         * @param mainRecordGuid The main record oid to remove the changes for
         * @param recordId The record oid to remove the changes for
         * @param toggleRelation True, if the contained relations should be switched to not-temporary instead
         *      (defaults to false)
         */
        RemoveChanges(mainRecordGuid: BA.Classes.Guid, recordId: BA.Classes.Guid, toggleRelation?: boolean): void;
        /**
         * Setter function for the LiveCheckEnabled property.
         *
         * @param enable boolean value to set
         */
        SetEnableLiveCheck(enable: boolean): void;
        /**
         *  Getter function for the LiveCheckEnabled property.
         *
         *  @returns true, if live check is enabled.
         */
        IsEnableLiveCheck(): boolean;
        /**
         * Queries the server interface for duplicates for the given record id and data source and calls the provided
         * callback functions with the information that has been retrieved.
         *
         * @param recordId the record to get the duplicates for
         * @param mainDataSource the data source of the record to be checked
         * @param onSuccess callback method to call when the check was successful (even if empty)
         * @param onFail callback method to call if the request failed for whatever causes
         */
        GetDuplicates(recordId: BA.Classes.Guid, mainDataSource: BA.Classes.Guid, onSuccess: Function, onFail: Function): void;
        /**
         * Function to open the duplicates dialog for the given record and data source.
         * Hint: The dialog can only be opened once.
         *
         * @param parentRecord The record oid of the parent record that has duplicates
         * @param mainDataSource The data source of the parent record that has duplicates
         */
        OpenDuplicatesDialog(parentRecord: BA.Classes.Guid, mainDataSource: BA.Classes.Guid, duplicateCount: number): void;
        /**
         * Updates the current toaster by re-searching for duplicates with the current parameters. This is needed, when
         * i.e. some record is marked as not-a-duplicate and thus the result would change.
         *
         * @param mainRecordGuid the record to re-search
         */
        ReSearch(mainRecordGuid: BA.Classes.Guid): void;
        /** Starts the check interval if it is not already running. */
        private StartCheckInterval;
        /** Function that is executed in an interval as long as there are dirty changes */
        private CheckIntervalFunction;
        /**
         * Called, when the garbage-can icon on a sub table entry is pressed in order to remove all changes on the
         * given record regarding the given sub table property and the given index. All following changes with a
         * greater index than the given will move to fill the gap.
         * Example with index = 1:
         *      EmailAddresses[1].EmailAddress will be deleted
         *      EmailAddresses[2].EmailAddress will become EmailAddresses[1].EmailAddress
         *
         * @param recordId The record to change in the existing changes
         * @param subTableProperty The name of the sub table record proeprty
         * @param index The index to remove
         */
        RemoveSubData(recordId: BA.Classes.Guid, subTableProperty: string, index: number): void;
        /**
         * Called, when the plus icon on a sub table entry is pressed in order to insert space at the given index.
         * This is done by moving all elements of the given record and sub table property one index further.
         * Example: EmailAddresses[3].EmailAddress will become EmailAddresses[4].EmailAddress
         *
         * @param recordId The record to change in the existing changes
         * @param subTableProperty The name of the sub table record proeprty
         * @param index The index to insert a space after
         */
        InsertSubData(recordId: BA.Classes.Guid, subTableProperty: string, index: number): void;
        /**
         * Callback the service controller in order to retrieve information about probable duplicates of the given
         * record. The current changes will be respected during the duplicate search.
         *
         * @param recordId the oid of the record to be checked
         * @param changes the current changes for that record
         */
        private CallbackServerController;
    }
    /** Class that contains all needed information about current changes to relevant fields or relations. */
    class DuplicateChangeInfo {
        mainRecordDataSource: BA.Classes.Guid;
        RecordId: BA.Classes.Guid;
        DataSource: BA.Classes.Guid;
        ElementName: string | BA.Classes.Guid;
        Value: string;
        Temporary: boolean;
        dirty: boolean;
        /** Function that is called when a change itself is changed. */
        static OnDirty: Function;
        /**
         * Static function that creates a copy of a given DuplicateChangeInfo object. The new object will not be dirty.
         *
         * @param from the object to copy values from
         * @returns a newly created DuplicateChangeInfo, containing the information of the source object
         */
        static CopyFrom(from: DuplicateChangeInfo): DuplicateChangeInfo;
        /**
         * Constructor of the change objects, which defines a couple of default properties that are automatically
         * available on the object.
         *
         * @param mainRecordDataSource data source of the main record that is changed (i.e. Company-guid)
         * @param RecordId the record oid of the record that actually is changed (probably an address or the company itself)
         * @param DataSource the data sourc of the record that actually is changed
         * @param ElementName the element that has been changed; this might either be a field name (without DLG-prefix)
         *          or the guid of some relation type
         * @param Value the new value of the changed element (new field value of new related record oid)
         * @param Temporary only used for relations; true, if the relation just has been created and not persisted yet
         * @param dirty true, if this is a new change (defaults to true)
         */
        constructor(mainRecordDataSource: BA.Classes.Guid, RecordId: BA.Classes.Guid, DataSource: BA.Classes.Guid, ElementName: string | BA.Classes.Guid, Value: string, Temporary: boolean, dirty?: boolean);
        /** Backing member for the Dirty property */
        private _Dirty;
        /**
         *  Getter for the Dirty property.
         *
         *  @returns Returns simply the content of the backing field
         */
        get Dirty(): boolean;
        /**
         * Setter for the Dirty property. If the value is true, the OnDirty-callback function will be called.
         *
         * @param value The value to set to the backing field
         */
        set Dirty(value: boolean);
    }
    /**
     * Model that is specificly used for transferring data to the server. It is nearly identical to the
     * DuplicateChangeInfo model but it does not call some OnDirty function when it gets dirty.
     */
    class DuplicateChangeInfoTransferModel {
        RecordId: string;
        DataSource: string;
        ElementName: string;
        Value: string;
        Temporary: boolean;
        dirty: boolean;
        /**
         * Create a TransferModel-copy of the given default DuplicateChangeInfo object.
         *
         * @param from DuplicateChangeInfo object to copy values from
         * @returns a new DuplicateChangeInfoTransferModel that has its values copied from the given
         *      DuplicateChangeInfo object
         */
        static CopyFrom(from: DuplicateChangeInfo): DuplicateChangeInfoTransferModel;
        /**
         * Constructor of the change objects, which defines a couple of default properties that are automatically
         * available on the object.
         *
         * @param RecordId the record oid of the record that actually is changed (probably an address or the company itself)
         * @param DataSource the data sourc of the record that actually is changed
         * @param ElementName the element that has been changed; this might either be a field name (without DLG-prefix)
         *          or the guid of some relation type
         * @param Value the new value of the changed element (new field value of new related record oid)
         * @param Temporary only used for relations; true, if the relation just has been created and not persisted yet
         * @param dirty true, if this is a new change (defaults to true)
         */
        constructor(RecordId: string, DataSource: string, ElementName: string, Value: string, Temporary: boolean, dirty?: boolean);
    }
}
