﻿using BA.API;
using BA.Core.DB;
using BA.Core.Enums;
using BA.Core.Enums.WorkManager;
using BA.Core.WorkManager;
using $safeprojectname$.DB;
using DevExpress.Xpo;
using System;
using System.Linq;

namespace $safeprojectname$.Worker
{
    /// <summary>
    /// Dieser Hintergrundprozess verarbeitet die selektierten Datensätze. Diese wurden vom Igniter in die temporäre Tabelle geschrieben. 
    /// Über die <see cref="TaskExecutionId"/> können die Datensätze ermittelt werden.
    /// 
    /// Wenn eine Verarbeitung von Datensätzen selbst implementiert wird. Muss beim Konzept beachtet werden, das der Prozess auf Fehler stoßen 
    /// und die Verarbeitung beispielsweise durch Herunterfahren der Anwendung unterbrochen werden kann.
    /// 
    /// In diesem Beispiel ist die zu verarbeitende Menge durch die Einträge in der temporären Tabelle definiert. Wenn ein Datensatz verarbeitet
    /// worden ist, wird der enstprechende Eintrag in der temporären Tabelle gelöscht.
    /// 
    /// Durch das Verwenden der <see cref="UnitOfWork"/> wird sichergestellt, das das Löschen des Eintrags und die Verarbeitung eine Transaktion ist.
    /// </summary>
    public class AddServiceWorker : WorkItemBase
    {
        // Id der Datensätze in der temporären Tabelle
        public Guid TaskExecutionId { get; set; }

        // Die zu setzende Bemerkung
        public string Remark { get; set; }

        // Das zu setzende Datum
        public DateTime Date { get; set; }

        // Zähler für Datensätze bei denen die Rechte fehlen
        public int MissingRights { get; set; }

        public AddServiceWorker() : base()
        {
            // Die erwartete Laufzeit ist land. Damit die entsprechende Queue genutzt wird.
            ExpectedRunTime = EnumExpectedRunTime.Long;
        }

        /// <summary>
        /// Implementierung des Prozesses. Wird bei Jedem Start aufgerufen
        /// </summary>
        protected override void Run()
        {
            // Abfrage auf die temporäre Tabelle
            IQueryable<OrmTempSelectedRecords> query = UnitOfWork.Query<OrmTempSelectedRecords>();
            query = query.Where(ff => ff.TaskExecutionId == TaskExecutionId);

            // Maximale Anzahl setzen. Wenn sie größer als 0 ist. Wird der Prozess neugestartet und der Count der Abfrage enthählt nicht 
            // mehr die schon verarbeiteten Datensätze.
            if (MaxProgress == 0)
                MaxProgress = query.Count();

            // Abfrage der zu verarbeitenden Maschinen
            IQueryable<Guid> oids = query.Select(ff => ff.SelectedRecordOid);
            IQueryable<OrmEngine> engines = Api.ORM.GetQuery<OrmEngine>(UnitOfWork).Where(ff => oids.Contains(ff.Oid));
            foreach (OrmEngine engine in engines)
            {
                // Abprüfung der Rechte
                if (engine.IsAllowed(EnumTableOperations.Edit))
                {
                    // Verarbeitung des aktuellen Datensatzes
                    OrmSubEngineServices service = engine.Services.AddNewObject();
                    service.SortOrder = engine.Services.Count() - 1;
                    service.ServiceDate = Date;
                    service.Remark = Remark;
                    engine.Save();
                }
                else
                    MissingRights++;

                // Löschen des temporären Eintrages
                query.Where(ff => ff.SelectedRecordOid == engine.Oid).FirstOrDefault()?.Delete();

                // Exception, wenn der dritte Datensatz im ersten Durchlauf verarbeitet wird.
                if (CurrentProgress == 2 && NumberOfStarts == 1)
                    throw new Exception(Logger.Translate("160C2DB4-E013-4569-8AB1-A37BDF776707"));

                // Zähler für die verarbeiteten Datensätze
                CurrentProgress++;
                Save();
            }
        }

        /// <summary>
        /// Prozess ist beendet und dem Anwender wird eine Nachricht zugestellt.
        /// </summary>
        protected override void WorkItemFinished()
        {
            if (State == EnumWorkItemState.Finished)
                if (MissingRights == 0)
                    // Der Anwender hatte auf alle Datensätze ausreichend Rechte
                    Api.ClientCommunication.CreateSuccess(Api.User.CurrentUserGuid(), "dd1ceadc-6808-4583-b6b7-dda73188b5a8".Translate(CurrentProgress));
                else
                    // Auf eineige Datensätze hatte der Anwender nicht genügend Rechte
                    Api.ClientCommunication.CreateSuccess(Api.User.CurrentUserGuid(), "e3f652e7-8019-4444-9e6c-f3c30528e862".Translate(CurrentProgress - MissingRights, MissingRights));

            base.WorkItemFinished();
        }
    }
}